@@ -1,10 +1,11 @@
-# -*- coding: utf-8 -*-
-""" PyCorrFit loader
-"""
+"""PyCorrFit loader"""
 from os.path import dirname, abspath, split
-
 import sys
+
 sys.path = [split(abspath(dirname(__file__)))[0]] + sys.path
 
-import pycorrfit
-pycorrfit.Main()
+from pycorrfit.gui import main
+
+
+if __name__ == "__main__":
+    main.Main()
