@@ -1,8 +1,4 @@
-# -*- coding: utf-8 -*-
-"""
-This file contains TIR one component models + Triplet
-"""
-import numpy as np                  # NumPy
+import numpy as np
 import scipy.special as sps
 
 def wixi(x):
